import os
import ntpath


class Cat2:
	"""Open file class"""

	def __init__(self, file_path: str, line_from: int=None, line_to: int=None):
		self._line_from = line_from
		if self._line_from:
			self._line_from -= 1
		self._line_to = line_to
		if self._line_to:
			self._line_to -= 1
		self._file_format = None
		self._file_path = file_path
		self._lines_count = None
		self._f = None
		self._i = 0

	def __iter__(self) -> "Cat2":
		"""Discover file format,
		set counter,
		set line from in file."""

		self._i = 0
		self._f = self._open_file()

		if self._line_from:
			for _ in range(int(self._line_from)):
				next(self._f)
			self._i = self._line_from

		return self

	def __next__(self) -> str:
		"""Watch for line to,
		set counter,
		return line."""

		if self._line_to:
			if self._i > self._line_to:
				raise StopIteration

		self._i += 1

		if self._file_format == 'b':
			return next(self._f)
		else:
			return next(self._f).replace('\n','')

	def __len__(self) -> int:
		"""Count lines."""

		if self._lines_count is None:
			with self._open_file() as f:
				self._lines_count = len(f.readlines())

		return self._lines_count

	def _open_file(self):
		"""Open the file"""

		if os.path.isdir(self._file_path):
			filename = ntpath.basename(self._file_path)
			raise Cat2Exception(f'{filename} is a directory!')

		if self._file_format is None:
			with open(self._file_path, 'r') as f:
				try:
					f.read(32)
					self._file_format = ''
				except UnicodeDecodeError:
					self._file_format = 'b'

		return open(self._file_path, f'r{self._file_format}')

	@property
	def line_number(self):
		"""Format number with leading zeros."""

		i = self._i
		number_of_digits_current_line = len(str(i))
		number_of_digits_file_len = len(str(len(self)))
		
		def add_zero(no, cl, fl):
			if fl > cl:
				no = f'0{no}'
				return add_zero(no, cl+1, fl)
			else:
				return no

		no = add_zero(str(i), 
					  number_of_digits_current_line,
					  number_of_digits_file_len)

		return no

	@property
	def is_bytes(self):
		"""Is it a bytes file"""

		return self._file_format == 'b'


class Cat2Exception(Exception):
	pass



import os
import re
import click

from cat2 import Cat2, Cat2Exception


@click.command()
@click.argument('file_path')
@click.option('-q', 'query', help="Query word")
@click.option('-f', 'line_from', type=int, help="Line from")
@click.option('-t', 'line_to', type=int, help="Line to")
def cli(file_path, query, line_from, line_to):

	rows, columns = os.popen('stty size', 'r').read().split()
	
	print('Analysing file...', end="\r")
	print('                 ')

	cat2_file = Cat2(file_path, line_from, line_to)
	
	try:
		if line_from is not None:
			if line_to:
				number_of_lines = line_to-line_from
			else:
				number_of_lines = len(cat2_file)-line_from
		else:
			number_of_lines = len(cat2_file)

		if number_of_lines > 200:
			click.secho(f'You are about to print {number_of_lines} lines.', fg='red')
			if not click.confirm('Do you want to continue?'):
				click.echo()
				return

		for line in cat2_file:
			click.secho(f' {cat2_file.line_number}. ', bg='blue', bold=True, nl=False)

			if query:
				if query in line:
					line_parts = line.split(query)
					line = click.style(line_parts[0], bg='red')
					line += click.style(query, bg='yellow', fg='black')
					line += click.style(line_parts[1], bg='red')
					line += click.style(' '*(int(columns)-len(line)), bg='red')
			
			click.echo(f'  {line}  ')

			if cat2_file.is_bytes:
				click.echo()
	except Cat2Exception as e:
		click.secho(str(e), fg='red')
		
	click.echo()


from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='cat2',
    version='0.3.1',

    description='Print file contents like a boss',
    long_description=readme(),
    long_description_content_type='text/markdown',

    url='https://github.com/arrrlo/cat2',
    licence='MIT',

    author='Ivan Arar',
    author_email='ivan.arar@gmail.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='cat, print, file, contents',

    packages=['cat2'],
    install_requires=[
        'click~=6.7',
        'colorama~=0.3'
    ],
    entry_points={
        'console_scripts': [
            'cat2=cat2.cli:cli'
        ],
    },

    project_urls={
        'Source': 'https://github.com/arrrlo/cat2',
    },
)
